@@ -16,7 +16,7 @@ __author__ = "Charles R Schmidt <schmidtc@gmail.com>"
 
 from struct import calcsize, unpack, pack
 
-from itertools import islice
+from itertools import izip, islice
 import array
 import sys
 
@@ -25,7 +25,7 @@ PY3 = int(sys.version[0]) > 2
 if PY3:
     import io
 else:
-    from io import StringIO
+    from cStringIO import StringIO
 
 if sys.byteorder == 'little':
     SYS_BYTE_ORDER = '<'
@@ -321,7 +321,7 @@ class shp_file:
     def type(self):
         return self.shape.String_Type
 
-    def __next__(self):
+    def next(self):
         """returns the next Shape in the shapeFile
 
         Example:
@@ -465,7 +465,7 @@ class shx_file:
         fmt = '>%di' % (2 * numRecords)
         size = calcsize(fmt)
         dat = unpack(fmt, self.fileObj.read(size))
-        self.index = [(dat[i] * 2, dat[i + 1] * 2) for i in range(
+        self.index = [(dat[i] * 2, dat[i + 1] * 2) for i in xrange(
             0, len(dat), 2)]
 
     def _create_shx_file(self):
@@ -634,7 +634,7 @@ class PolyLine:
         #record['Vertices'] = [(record['Vertices'][i],record['Vertices'][i+1]) for i in xrange(0,record['NumPoints']*2,2)]
         verts = record['Vertices']
         #Next line is equivalent to: zip(verts[::2],verts[1::2])
-        record['Vertices'] = list(zip(
+        record['Vertices'] = list(izip(
             islice(verts, 0, None, 2), islice(verts, 1, None, 2)))
         if not record['Parts Index']:
             record['Parts Index'] = [0]
@@ -683,7 +683,7 @@ class PolyLineZ(object):
                          ('Marray', ('d', record['NumPoints']), '<'),)
         _unpackDict2(record, contentStruct, dat)
         verts = record['Vertices']
-        record['Vertices'] = list(zip(
+        record['Vertices'] = list(izip(
             islice(verts, 0, None, 2), islice(verts, 1, None, 2)))
         if not record['Parts Index']:
             record['Parts Index'] = [0]
@@ -733,6 +733,7 @@ class Polygon(PolyLine):
 
 
 class MultiPoint:
+    String_Type = 'MULTIPOINT'
     def __init__(self):
         raise NotImplementedError("No MultiPoint Support at this time.")
 
@@ -742,31 +743,37 @@ class PolygonZ(PolyLineZ):
 
 
 class MultiPointZ:
+    String_Type = 'MULTIPOINTZ'
     def __init__(self):
         raise NotImplementedError("No MultiPointZ Support at this time.")
 
 
 class PointM:
+    String_Type = 'POINTM'
     def __init__(self):
         raise NotImplementedError("No PointM Support at this time.")
 
 
 class PolyLineM:
+    String_Type = 'ARCM'
     def __init__(self):
         raise NotImplementedError("No PolyLineM Support at this time.")
 
 
 class PolygonM:
+    String_Type = 'POLYGONM'
     def __init__(self):
         raise NotImplementedError("No PolygonM Support at this time.")
 
 
 class MultiPointM:
+    String_Type = 'MULTIPOINTM'
     def __init__(self):
         raise NotImplementedError("No MultiPointM Support at this time.")
 
 
 class MultiPatch:
+    String_Type = 'MULTIPATCH'
     def __init__(self):
         raise NotImplementedError("No MultiPatch Support at this time.")
 
