@@ -102,7 +102,7 @@ class Moran(object):
     >>> w = pysal.open(pysal.examples.get_path("stl.gal")).read()
     >>> f = pysal.open(pysal.examples.get_path("stl_hom.txt"))
     >>> y = np.array(f.by_col['HR8893'])
-    >>> mi = Moran(y,  w)
+    >>> mi = pysal.Moran(y,  w)
     >>> "%7.5f" % mi.I
     '0.24366'
     >>> mi.EI
@@ -114,6 +114,7 @@ class Moran(object):
 
     >>> w = pysal.open(pysal.examples.get_path("sids2.gal")).read()
     >>> f = pysal.open(pysal.examples.get_path("sids2.dbf"))
+    >>> import numpy as np
     >>> SIDR = np.array(f.by_col("SIDR74"))
     >>> mi = pysal.Moran(SIDR,  w)
     >>> "%6.4f" % mi.I
@@ -129,6 +130,15 @@ class Moran(object):
     >>> mi_1.p_norm
     5.7916539074498452e-05
 
+    Example from http://www.lpc.uottawa.ca/publications/moransi/moran.htm
+
+    >>> w = pysal.lat2W(3, 3)
+    >>> y = np.arange(1, 10)
+    >>> mi = pysal.Moran(y, w, transformation='B')
+    >>> mi.VI_rand
+    0.059687500000000004
+    >>> mi.VI_norm
+    0.053125000000000006
     """
     def __init__(self, y, w, transformation="r", permutations=PERMUTATIONS,
                  two_tailed=True):
@@ -155,7 +165,7 @@ class Moran(object):
 
         if permutations:
             sim = [self.__calc(np.random.permutation(self.z))
-                   for i in range(permutations)]
+                   for i in xrange(permutations)]
             self.sim = sim = np.array(sim)
             above = sim >= self.I
             larger = above.sum()
@@ -179,22 +189,28 @@ class Moran(object):
         self.z2ss = (z * z).sum()
         self.EI = -1. / (self.n - 1)
         n = self.n
+        n2 = n * n
         s1 = self.w.s1
         s0 = self.w.s0
         s2 = self.w.s2
         s02 = s0 * s0
-        v_num = n * n * s1 - n * s2 + 3 * s0 * s0
-        v_den = (n - 1) * (n + 1) * s0 * s0
+        v_num = n2  * s1 - n * s2 + 3 * s02
+        v_den = (n - 1) * (n + 1) * s02
         self.VI_norm = v_num / v_den - (1.0 / (n - 1)) ** 2
         self.seI_norm = self.VI_norm ** (1 / 2.)
 
-        k = (1 / ((z ** 4).sum()) * (((z ** 2).sum()) ** 2))
-        vi = (1 / (((n - 1) ** 3) * s02)) * ((n * ((n * n - 3 * n + 3)
-                                                   * s1 - n * s2 + 3 * s02))
-                                             - (k * ((n * n - n) * s1 - 2 * n *
-                                                     s2 + 6 * s02)))
-        self.VI_rand = vi
-        self.seI_rand = vi ** (1 / 2.)
+        # variance under randomization
+        xd4 = z**4
+        xd2 = z**2
+        k_num = xd4.sum() / n
+        k_den = (xd2.sum() / n)**2
+        k = k_num / k_den
+        EI = self.EI
+        A = n * ((n2 - 3 * n + 3) * s1 - n * s2 + 3 * s02)
+        B = k * ((n2 - n) * s1 - 2 * n * s2 + 6 * s02  )
+        VIR = (A - B) / ((n - 1) * (n - 2) * (n - 3 ) * s02) - EI*EI
+        self.VI_rand = VIR 
+        self.seI_rand = VIR ** (1 / 2.)
 
     def __calc(self, z):
         zl = slag(self.w, z)
@@ -365,7 +381,7 @@ class Moran_BV(object):
         self.I = self.__calc(zy)
         if permutations:
             nrp = np.random.permutation
-            sim = [self.__calc(nrp(zy)) for i in range(permutations)]
+            sim = [self.__calc(nrp(zy)) for i in xrange(permutations)]
             self.sim = sim = np.array(sim)
             above = sim >= self.I
             larger = above.sum()
@@ -496,7 +512,7 @@ def Moran_BV_matrix(variables, w, permutations=0, varnames=None):
     """
 
     k = len(variables)
-    rk = list(range(0, k - 1))
+    rk = range(0, k - 1)
     results = {}
     for i in rk:
         for j in range(i + 1, k):
@@ -695,7 +711,7 @@ class Moran_Rate(Moran):
                  for e,pop,adj in zip(events, populations, adjusted)]
         names = ['-'.join((e,p)) for e,p in zip(events, populations)]
         out_df = df.copy()
-        rate_df = out_df.from_items(list(zip(names, rates))) #trick to avoid importing pandas
+        rate_df = out_df.from_items(zip(names, rates)) #trick to avoid importing pandas
         stat_df = _univariate_handler(rate_df, names, w=w, inplace=False, 
                                       pvalue = pvalue, outvals = outvals, 
                                       swapname=swapname, 
@@ -855,7 +871,7 @@ class Moran_Local(object):
         z = self.z
         lisas = np.zeros((self.n, self.permutations))
         n_1 = self.n - 1
-        prange = list(range(self.permutations))
+        prange = range(self.permutations)
         k = self.w.max_neighbors + 1
         nn = self.n - 1
         rids = np.array([np.random.permutation(nn)[0:k] for i in prange])
@@ -864,7 +880,7 @@ class Moran_Local(object):
         w = [self.w.weights[ido[i]] for i in ids]
         wc = [self.w.cardinalities[ido[i]] for i in ids]
 
-        for i in range(self.w.n):
+        for i in xrange(self.w.n):
             idsi = ids[ids != i]
             np.random.shuffle(idsi)
             tmp = z[idsi[rids[:, 0:wc[i]]]]
@@ -1092,7 +1108,7 @@ class Moran_Local_BV(object):
         """
         lisas = np.zeros((self.n, self.permutations))
         n_1 = self.n - 1
-        prange = list(range(self.permutations))
+        prange = range(self.permutations)
         k = self.w.max_neighbors + 1
         nn = self.n - 1
         rids = np.array([np.random.permutation(nn)[0:k] for i in prange])
@@ -1103,7 +1119,7 @@ class Moran_Local_BV(object):
 
         zx = self.zx
         zy = self.zy
-        for i in range(self.w.n):
+        for i in xrange(self.w.n):
             idsi = ids[ids != i]
             np.random.shuffle(idsi)
             tmp = zy[idsi[rids[:, 0:wc[i]]]]
@@ -1361,7 +1377,7 @@ class Moran_Local_Rate(Moran_Local):
                  for e,pop,adj in zip(events, populations, adjusted)]
         names = ['-'.join((e,p)) for e,p in zip(events, populations)]
         out_df = df.copy()
-        rate_df = out_df.from_items(list(zip(names, rates))) #trick to avoid importing pandas
+        rate_df = out_df.from_items(zip(names, rates)) #trick to avoid importing pandas
         _univariate_handler(rate_df, names, w=w, inplace=True, 
                                       pvalue = pvalue, outvals = outvals, 
                                       swapname=swapname,
