@@ -1,4 +1,4 @@
-
+from __future__ import division
 """
 Apply smoothing to rate computation
 
@@ -23,7 +23,6 @@ from ..cg import get_angle_between, get_points_dist, get_segment_point_dist,\
 from ..common import np, KDTree, requires as _requires
 from ..weights.spatial_lag import lag_spatial as slag
 from scipy.stats import gamma, norm, chi2, poisson
-from functools import reduce
 
 __all__ = ['Excess_Risk', 'Empirical_Bayes', 'Spatial_Empirical_Bayes', 'Spatial_Rate', 'Kernel_Smoother', 'Age_Adjusted_Smoother', 'Disk_Smoother', 'Spatial_Median_Rate', 'Spatial_Filtering', 'Headbanging_Triples', 'Headbanging_Median_Rate', 'flatten', 'weighted_median', 'sum_by_n', 'crude_age_standardization', 'direct_age_standardization', 'indirect_age_standardization', 'standardized_mortality_ratio', 'choynowski', 'assuncao_rate']
 
@@ -102,7 +101,7 @@ def weighted_median(d, w):
 
     """
     dtype = [('w', '%s' % w.dtype), ('v', '%s' % d.dtype)]
-    d_w = np.array(list(zip(w, d)), dtype=dtype)
+    d_w = np.array(zip(w, d), dtype=dtype)
     d_w.sort(order='v')
     reordered_w = d_w['w'].cumsum()
     cumsum_threshold = reordered_w[-1] * 1.0 / 2
@@ -371,7 +370,7 @@ def indirect_age_standardization(e, b, s_e, s_b, n, alpha=0.05):
     log_smr_upper = log_smr + norm_thres * log_smr_sd
     smr_lower = np.exp(log_smr_lower) * s_r_all
     smr_upper = np.exp(log_smr_upper) * s_r_all
-    res = list(zip(adjusted_r, smr_lower, smr_upper))
+    res = zip(adjusted_r, smr_lower, smr_upper)
     return res
 
 
@@ -550,17 +549,18 @@ def assuncao_rate(e, b):
     ebi_b = e_sum * 1.0 / b_sum
     s2 = sum(b * ((y - ebi_b) ** 2)) / b_sum
     ebi_a = s2 - ebi_b / (float(b_sum) / len(e))
-    ebi_v = ebi_a + ebi_b / b
+    ebi_v_raw = ebi_a + ebi_b / b
+    ebi_v = np.where(ebi_v_raw < 0, ebi_b / b, ebi_v_raw)
     return (y - ebi_b) / np.sqrt(ebi_v)
 
 class _Smoother(object):
     """
     This is a helper class that implements things that all smoothers should do.
-    Right now, the only thing that we need to propagate is the by_col function. 
+    Right now, the only thing that we need to propagate is the by_col function.
 
     TBQH, most of these smoothers should be functions, not classes (aside from
     maybe headbanging triples), since they're literally only inits + one
-    attribute. 
+    attribute.
     """
     def __init__(self):
         pass
@@ -568,7 +568,7 @@ class _Smoother(object):
     @classmethod
     def by_col(cls, df, e,b, inplace=False, **kwargs):
         """
-        Compute smoothing by columns in a dataframe. 
+        Compute smoothing by columns in a dataframe.
 
         Parameters
         -----------
@@ -583,15 +583,15 @@ class _Smoother(object):
         inplace :  bool
                    a flag denoting whether to output a copy of `df` with the
                    relevant smoothed columns appended, or to append the columns
-                   directly to `df` itself. 
+                   directly to `df` itself.
         **kwargs:  optional keyword arguments
                    optional keyword options that are passed directly to the
-                   smoother. 
+                   smoother.
 
         Returns
         ---------
         a copy of `df` containing the columns. Or, if `inplace`, this returns
-        None, but implicitly adds columns to `df`.  
+        None, but implicitly adds columns to `df`.
         """
         if not inplace:
             new = df.copy()
@@ -719,13 +719,13 @@ class Empirical_Bayes(_Smoother):
 class _Spatial_Smoother(_Smoother):
     """
     This is a helper class that implements things that all the things that
-    spatial smoothers should do. 
+    spatial smoothers should do.
     .
-    Right now, the only thing that we need to propagate is the by_col function. 
+    Right now, the only thing that we need to propagate is the by_col function.
 
     TBQH, most of these smoothers should be functions, not classes (aside from
     maybe headbanging triples), since they're literally only inits + one
-    attribute. 
+    attribute.
     """
     def __init__(self):
         pass
@@ -733,7 +733,7 @@ class _Spatial_Smoother(_Smoother):
     @classmethod
     def by_col(cls, df, e,b, w=None, inplace=False, **kwargs):
         """
-        Compute smoothing by columns in a dataframe. 
+        Compute smoothing by columns in a dataframe.
 
         Parameters
         -----------
@@ -749,19 +749,19 @@ class _Spatial_Smoother(_Smoother):
                    the spatial weights object or objects to use with the
                    event-population pairs. If not provided and a weights object
                    is in the dataframe's metadata, that weights object will be
-                   used. 
+                   used.
         inplace :  bool
                    a flag denoting whether to output a copy of `df` with the
                    relevant smoothed columns appended, or to append the columns
-                   directly to `df` itself. 
+                   directly to `df` itself.
         **kwargs:  optional keyword arguments
                    optional keyword options that are passed directly to the
-                   smoother. 
+                   smoother.
 
         Returns
         ---------
         a copy of `df` containing the columns. Or, if `inplace`, this returns
-        None, but implicitly adds columns to `df`.  
+        None, but implicitly adds columns to `df`.
         """
         if not inplace:
             new = df.copy()
@@ -859,7 +859,7 @@ class Spatial_Empirical_Bayes(_Spatial_Smoother):
         ngh_num = np.ones_like(e)
         bi = slag(w, b) + b
         for i, idv in enumerate(w.id_order):
-            ngh = list(w[idv].keys()) + [idv]
+            ngh = w[idv].keys() + [idv]
             nghi = [w.id2i[k] for k in ngh]
             ngh_num[i] = len(nghi)
             v = sum(np.square(rate[nghi] - r_mean[i]) * b[nghi])
@@ -1073,12 +1073,12 @@ class Age_Adjusted_Smoother(_Spatial_Smoother):
         r = direct_age_standardization(e_n, b_n, s, w.n, alpha=alpha)
         self.r = np.array([i[0] for i in r])
         w.transform = 'o'
-    
+
     @_requires('pandas')
     @classmethod
     def by_col(cls, df, e,b, w=None, s=None, **kwargs):
         """
-        Compute smoothing by columns in a dataframe. 
+        Compute smoothing by columns in a dataframe.
 
         Parameters
         -----------
@@ -1094,22 +1094,22 @@ class Age_Adjusted_Smoother(_Spatial_Smoother):
                    the spatial weights object or objects to use with the
                    event-population pairs. If not provided and a weights object
                    is in the dataframe's metadata, that weights object will be
-                   used. 
+                   used.
         s       :  string or list of strings
                    the name or names of columns to use as a standard population
-                   variable for the events `e` and at-risk populations `b`. 
+                   variable for the events `e` and at-risk populations `b`.
         inplace :  bool
                    a flag denoting whether to output a copy of `df` with the
                    relevant smoothed columns appended, or to append the columns
-                   directly to `df` itself. 
+                   directly to `df` itself.
         **kwargs:  optional keyword arguments
                    optional keyword options that are passed directly to the
-                   smoother. 
+                   smoother.
 
         Returns
         ---------
         a copy of `df` containing the columns. Or, if `inplace`, this returns
-        None, but implicitly adds columns to `df`.  
+        None, but implicitly adds columns to `df`.
         """
         if s is None:
             raise Exception('Standard population variable "s" must be supplied.')
@@ -1158,7 +1158,7 @@ class Age_Adjusted_Smoother(_Spatial_Smoother):
             max_len = 0 if len(this_r) > max_len else max_len
             rdf.append((outcol, this_r.tolist()))
         padded = (r[1] + [None] * max_len for r in rdf)
-        rdf = list(zip((r[0] for r in rdf), padded))
+        rdf = zip((r[0] for r in rdf), padded)
         rdf = pd.DataFrame.from_items(rdf)
         return rdf
 
@@ -1327,7 +1327,7 @@ class Spatial_Median_Rate(_Spatial_Smoother):
         if not w.id_order_set:
             raise ValueError("w id_order must be set to align with the order of e and b")
         e = np.asarray(e).flatten()
-        b = np.asarray(b).flatten() 
+        b = np.asarray(b).flatten()
         self.r = e * 1.0 / b
         self.aw, self.w = aw, w
         while iteration:
@@ -1450,7 +1450,7 @@ class Spatial_Filtering(_Smoother):
         y_range = bbox[1][1] - bbox[0][1]
         x, y = np.mgrid[bbox[0][0]:bbox[1][0]:float(x_range) / x_grid,
                         bbox[0][1]:bbox[1][1]:float(y_range) / y_grid]
-        self.grid = list(zip(x.ravel(), y.ravel()))
+        self.grid = zip(x.ravel(), y.ravel())
         self.r = []
         if r is None and pop is None:
             raise ValueError("Either r or pop should not be None")
@@ -1470,13 +1470,13 @@ class Spatial_Filtering(_Smoother):
                     b_n_f = b_n[[0]]
                 self.r.append(e_n_f[-1] * 1.0 / b_n_f[-1])
         self.r = np.array(self.r)
-    
+
     @_requires('pandas')
     @classmethod
     def by_col(cls, df, e, b, x_grid, y_grid, geom_col='geometry', **kwargs):
         """
         Compute smoothing by columns in a dataframe. The bounding box and point
-        information is computed from the geometry column. 
+        information is computed from the geometry column.
 
         Parameters
         -----------
@@ -1494,13 +1494,13 @@ class Spatial_Filtering(_Smoother):
                    number of grid cells to use along the y-axis
         geom_col:  string
                    the name of the column in the dataframe containing the
-                   geometry information. 
+                   geometry information.
         **kwargs:  optional keyword arguments
                    optional keyword options that are passed directly to the
-                   smoother. 
+                   smoother.
         Returns
         ---------
-        a new dataframe of dimension (x_grid*y_grid, 3), containing the 
+        a new dataframe of dimension (x_grid*y_grid, 3), containing the
         coordinates of the grid cells and the rates associated with those grid
         cells.
         """
@@ -1526,13 +1526,16 @@ class Spatial_Filtering(_Smoother):
             grid = np.asarray(r.grid).reshape(-1,2)
             name = '_'.join(('-'.join((ename, bname)), cls.__name__.lower()))
             colnames = ('_'.join((name, suffix)) for suffix in ['X', 'Y', 'R'])
-            items = [(name, col) for name,col in zip(colnames, [grid[:,0], 
-                                                                grid[:,1], 
+            items = [(name, col) for name,col in zip(colnames, [grid[:,0],
+                                                                grid[:,1],
                                                                 r.r])]
             res.append(pd.DataFrame.from_items(items))
         outdf = pd.concat(res)
         return outdf
 
+
+
+import warnings
 class Headbanging_Triples(object):
     """Generate a pseudo spatial weights instance that contains headbanging triples
 
@@ -1662,6 +1665,7 @@ class Headbanging_Triples(object):
     (0.33753, 0.302707)
     """
     def __init__(self, data, w, k=5, t=3, angle=135.0, edgecor=False):
+        raise DepricationWarning('Depricated')
         if k < 3:
             raise ValueError("w should be NeareastNeighbors instance & the number of neighbors should be more than 3.")
         if not w.id_order_set:
@@ -1669,39 +1673,39 @@ class Headbanging_Triples(object):
         self.triples, points = {}, {}
         for i, pnt in enumerate(data):
             ng = w.neighbor_offsets[i]
-            points[(i, Point(pnt))] = dict(list(zip(ng, [Point(d)
-                                                    for d in data[ng]])))
-        for i, pnt in list(points.keys()):
+            points[(i, Point(pnt))] = dict(zip(ng, [Point(d)
+                                                    for d in data[ng]]))
+        for i, pnt in points.keys():
             ng = points[(i, pnt)]
             tr, tr_dis = {}, []
-            for c in comb(list(ng.keys()), 2):
+            for c in comb(ng.keys(), 2):
                 p2, p3 = ng[c[0]], ng[c[-1]]
                 ang = get_angle_between(Ray(pnt, p2), Ray(pnt, p3))
                 if ang > angle or (ang < 0.0 and ang + 360 > angle):
                     tr[tuple(c)] = (p2, p3)
             if len(tr) > t:
-                for c in list(tr.keys()):
+                for c in tr.keys():
                     p2, p3 = tr[c]
                     tr_dis.append((get_segment_point_dist(
                         LineSegment(p2, p3), pnt), c))
                 tr_dis = sorted(tr_dis)[:t]
                 self.triples[i] = [trp for dis, trp in tr_dis]
             else:
-                self.triples[i] = list(tr.keys())
+                self.triples[i] = tr.keys()
         if edgecor:
             self.extra = {}
-            ps = dict([(p, i) for i, p in list(points.keys())])
-            chull = convex_hull(list(ps.keys()))
+            ps = dict([(p, i) for i, p in points.keys()])
+            chull = convex_hull(ps.keys())
             chull = [p for p in chull if len(self.triples[ps[p]]) == 0]
             for point in chull:
                 key = (ps[point], point)
                 ng = points[key]
-                ng_dist = [(get_points_dist(point, p), p) for p in list(ng.values())]
+                ng_dist = [(get_points_dist(point, p), p) for p in ng.values()]
                 ng_dist_s = sorted(ng_dist, reverse=True)
                 extra = None
                 while extra is None and len(ng_dist_s) > 0:
                     p2 = ng_dist_s.pop()[-1]
-                    p3s = list(ng.values())
+                    p3s = ng.values()
                     p3s.remove(p2)
                     for p3 in p3s:
                         dist_p2_p3 = get_points_dist(p2, p3)
@@ -1807,6 +1811,7 @@ class Headbanging_Median_Rate(object):
     array([ 0.00091659,  0.        ,  0.00156838,  0.0018315 ,  0.00498891])
     """
     def __init__(self, e, b, t, aw=None, iteration=1):
+        warnings.warn("Deprecated", DepricationWarning)
         self.r = e * 1.0 / b
         self.tr, self.aw = t.triples, aw
         if hasattr(t, 'extra'):
@@ -1822,9 +1827,9 @@ class Headbanging_Median_Rate(object):
         if hasattr(self, 'extra') and id in self.extra:
             extra = self.extra
             trp_r = r[list(triples[0])]
-            # observed rate 
+            # observed rate
             # plus difference in rate scaled by ratio of extrapolated distance
-            # & observed distance. 
+            # & observed distance.
             trp_r[-1] = trp_r[0] + (trp_r[0] - trp_r[-1]) * (
                 extra[id][-1] * 1.0 / extra[id][1])
             trp_r = sorted(trp_r)
@@ -1849,18 +1854,18 @@ class Headbanging_Median_Rate(object):
                 trp_r = r[list(trp)]
                 dtype = [('r', '%s' % trp_r.dtype), ('w',
                                                      '%s' % self.aw.dtype)]
-                trp_r = np.array(list(zip(trp_r, list(trp))), dtype=dtype)
+                trp_r = np.array(zip(trp_r, list(trp)), dtype=dtype)
                 trp_r.sort(order='r')
                 lowest.append(trp_r['r'][0])
                 highest.append(trp_r['r'][-1])
-                lowest_aw.append(self.aw[trp_r['w'][0]])
-                highest_aw.append(self.aw[trp_r['w'][-1]])
+                lowest_aw.append(self.aw[int(trp_r['w'][0])])
+                highest_aw.append(self.aw[int(trp_r['w'][-1])])
             wm_lowest = weighted_median(np.array(lowest), np.array(lowest_aw))
             wm_highest = weighted_median(
                 np.array(highest), np.array(highest_aw))
             triple_members = flatten(triples, unique=False)
             return r[id], wm_lowest, wm_highest, self.aw[id] * len(triples), self.aw[triple_members].sum()
-    
+
     def __get_median_from_screens(self, screens):
         if isinstance(screens, float):
             return screens
@@ -1880,18 +1885,18 @@ class Headbanging_Median_Rate(object):
     def __search_headbanging_median(self):
         r, tr = self.r, self.tr
         new_r = []
-        for k in list(tr.keys()):
+        for k in tr.keys():
             screens = self.__get_screens(
                 k, tr[k], weighted=(self.aw is not None))
             new_r.append(self.__get_median_from_screens(screens))
         self.r = np.array(new_r)
-    
+
     @_requires('pandas')
     @classmethod
     def by_col(cls, df, e, b, t=None, geom_col='geometry', inplace=False, **kwargs):
         """
         Compute smoothing by columns in a dataframe. The bounding box and point
-        information is computed from the geometry column. 
+        information is computed from the geometry column.
 
         Parameters
         -----------
@@ -1905,22 +1910,22 @@ class Headbanging_Median_Rate(object):
                    variables to be smoothed
         t       :  Headbanging_Triples instance or list of Headbanging_Triples
                    list of headbanging triples instances. If not provided, this
-                   is computed from the geometry column of the dataframe. 
+                   is computed from the geometry column of the dataframe.
         geom_col:  string
                    the name of the column in the dataframe containing the
-                   geometry information. 
+                   geometry information.
         inplace :  bool
                    a flag denoting whether to output a copy of `df` with the
                    relevant smoothed columns appended, or to append the columns
-                   directly to `df` itself. 
+                   directly to `df` itself.
         **kwargs:  optional keyword arguments
                    optional keyword options that are passed directly to the
-                   smoother. 
+                   smoother.
         Returns
         ---------
         a new dataframe containing the smoothed Headbanging Median Rates for the
         event/population pairs. If done inplace, there is no return value and
-        `df` is modified in place. 
+        `df` is modified in place.
         """
         import pandas as pd
         if not inplace:
@@ -1940,7 +1945,7 @@ class Headbanging_Median_Rate(object):
 
         #Headbanging_Triples doesn't take **kwargs, so filter its arguments
         # (self, data, w, k=5, t=3, angle=135.0, edgecor=False):
-        
+
         w = kwargs.pop('w', None)
         if w is None:
             found = False
@@ -1952,7 +1957,7 @@ class Headbanging_Median_Rate(object):
                 raise Exception('Weights not provided and no weights attached to frame!'
                                     ' Please provide a weight or attach a weight to the'
                                     ' dataframe')
-        
+
         k = kwargs.pop('k', 5)
         t = kwargs.pop('t', 3)
         angle = kwargs.pop('angle', 135.0)
@@ -1960,7 +1965,7 @@ class Headbanging_Median_Rate(object):
 
         hbt = Headbanging_Triples(data, w, k=k, t=t, angle=angle,
                                   edgecor=edgecor)
-        
+
         res = []
         for ename, bname in zip(e, b):
             r = cls(df[ename], df[bname], hbt, **kwargs).r
