@@ -26,10 +26,10 @@ class Geary_Tester(unittest.TestCase):
         self.assertAlmostEqual(c.z_norm, -3.7399778367629564)
         self.assertAlmostEqual(c.seC_norm, 0.17834040553138225)
 
-        self.assertAlmostEqual(c.VC_rand, 0.018437747611029367)
-        self.assertAlmostEqual(c.p_rand, 4.5059156794646782e-07)
-        self.assertAlmostEqual(c.z_rand, -4.9120733751216008)
-        self.assertAlmostEqual(c.seC_rand, 0.13578566791465646)
+        self.assertAlmostEqual(c.VC_rand,0.033411917666958356)
+        self.assertAlmostEqual(c.p_rand,0.00013165646189214729)
+        self.assertAlmostEqual(c.z_rand, -3.6489513837253944)
+        self.assertAlmostEqual(c.seC_rand, 0.18278927120309429)
 
         np.random.seed(12345)
         c = geary.Geary(self.y, self.w, permutations=999)
@@ -41,10 +41,10 @@ class Geary_Tester(unittest.TestCase):
         self.assertAlmostEqual(c.z_norm, -3.7399778367629564)
         self.assertAlmostEqual(c.seC_norm, 0.17834040553138225)
 
-        self.assertAlmostEqual(c.VC_rand, 0.018437747611029367)
-        self.assertAlmostEqual(c.p_rand, 4.5059156794646782e-07)
-        self.assertAlmostEqual(c.z_rand, -4.9120733751216008)
-        self.assertAlmostEqual(c.seC_rand, 0.13578566791465646)
+        self.assertAlmostEqual(c.VC_rand, 0.033411917666958356)
+        self.assertAlmostEqual(c.p_rand, 0.00013165646189214729)
+        self.assertAlmostEqual(c.z_rand, -3.6489513837253944)
+        self.assertAlmostEqual(c.seC_rand, 0.18278927120309429)
 
         self.assertAlmostEqual(c.EC_sim, 0.9980676303238214)
         self.assertAlmostEqual(c.VC_sim, 0.034430408799858946)
