@@ -135,6 +135,11 @@ class TestNetworkUtils(unittest.TestCase):
         self.distance, self.pred = util.dijkstra(self.ntw, self.ntw.edge_lengths, 0)
         self.assertAlmostEqual(self.distance[196], 5505.668247, places=4)
         self.assertEqual(self.pred[196], 133)
+        
+    def test_dijkstra_mp(self):
+        self.distance, self.pred = util.dijkstra_mp((self.ntw, self.ntw.edge_lengths, 0))
+        self.assertAlmostEqual(self.distance[196], 5505.668247, places=4)
+        self.assertEqual(self.pred[196], 133)
 
 if __name__ == '__main__':
     unittest.main()
